@@ -1,6 +1,7 @@
-/* Copyright (c) 1997-2018
-   Ewgenij Gawrilow, Michael Joswig (Technische Universitaet Berlin, Germany)
-   http://www.polymake.org
+/* Copyright (c) 1997-2019
+   Ewgenij Gawrilow, Michael Joswig, and the polymake team
+   Technische Universität Berlin, Germany
+   https://polymake.org
 
    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -33,7 +34,8 @@ Map<Rational, Rational> sum_of_square_roots_naive(const Array<Rational>& a)
 
    // Now start the summation, counting how many equal values there are in a row
    Map<Rational, Rational> coefficient_of_sqrt;
-   Entire<Array<Rational> >::const_iterator a1 = entire(sorted_a), a2 = a1;
+   auto a1 = entire(sorted_a);
+   auto a2 = a1;
    while (!a2.at_end()) {
       int multiplicity(1); 
       ++a2;
