@@ -1,6 +1,7 @@
-#  Copyright (c) 1997-2015
-#  Ewgenij Gawrilow, Michael Joswig (Technische Universitaet Berlin, Germany)
-#  http://www.polymake.org
+#  Copyright (c) 1997-2019
+#  Ewgenij Gawrilow, Michael Joswig, and the polymake team
+#  Technische Universität Berlin, Germany
+#  https://polymake.org
 #
 #  This program is free software; you can redistribute it and/or modify it
 #  under the terms of the GNU General Public License as published by the
@@ -15,19 +16,17 @@
 
 use strict;
 use namespaces;
+use warnings qw(FATAL void syntax misc);
 
 package Polymake::Core::UserSettings;
 
-# code from other modules to run after all settings loaded
-declare @add_custom_vars;
-
 ########################################################################
 #
 # Analyze the configuration mode and load the files if necessary.
 #
 
 sub init {
-   my $mode=shift;
+   my ($mode, @custom_blocks) = @_;
    if (Application::RuleFilter::allow_config($mode)) {
       foreach (split /;/, $mode) {
          if ($_ eq "user") {
@@ -37,10 +36,10 @@ sub init {
             ### TODO: select MacOS-specific default location if $^O eq "darwin" ?
             $PrivateDir=$ENV{POLYMAKE_USER_DIR} || "$ENV{HOME}/.polymake";
             next;
-         } 
-		 if ( $_ eq '@interactive' ) {
-			 require Polymake::Core::InteractiveHelp;
-			 next;
+         }
+         if ($_ eq '@interactive') {
+            require Polymake::Core::InteractiveHelp;
+            next;
          }
          my $user= s/^user=//;
          my $location;
@@ -104,8 +103,7 @@ sub init {
       }
    }
 
-   $_->() for @add_custom_vars;
-   @add_custom_vars=();
+   $_->() for @custom_blocks;
 }
 
 1
