@@ -1,6 +1,7 @@
-#  Copyright (c) 1997-2015
-#  Ewgenij Gawrilow, Michael Joswig (Technische Universitaet Berlin, Germany)
-#  http://www.polymake.org
+#  Copyright (c) 1997-2019
+#  Ewgenij Gawrilow, Michael Joswig, and the polymake team
+#  Technische Universität Berlin, Germany
+#  https://polymake.org
 #
 #  This program is free software; you can redistribute it and/or modify it
 #  under the terms of the GNU General Public License as published by the
@@ -16,6 +17,7 @@
 use strict;
 use mro;
 use namespaces;
+use warnings qw(FATAL void syntax misc);
 
 package Polymake::Struct;
 use Polymake::Ext;
@@ -67,7 +69,7 @@ sub import {
                "\n seems to be established via plain \@ISA assignment and conflicts with Struct declaration";
       }
    }
-   my $symtab=get_pkg($pkg);
+   my $symtab=get_symtab($pkg);
    my $constructor="";
    my $constructor_deferred="";
    my $alt_constructor_name;
@@ -102,7 +104,7 @@ sub import {
                $merger &&= [ @$merger ];
                $redefine=1;
                namespaces::using($pkg, $super) if $with_namespaces !~ /^no/;
-               $super_symtab=get_pkg($super);
+               $super_symtab=get_symtab($super);
                last;
             }
          }
@@ -236,7 +238,7 @@ sub import {
                 }
                 my %options=splice @$field, 2;
                 $init_deferred=$set_filter_is_method= $expr =~ /(?<!\\)\$this\b/;
-                if ($expr =~ s{(?: ^ | [\s,(]) \K \#(?:([\d+]) | (%)) (?= $ | [\s,)])}
+                if ($expr =~ s{(?: ^ | [\s,([{]) \K \#(?:([\d+]) | (%)) (?= $ | [-\s,)\]}])}
                               { $2 ? $keyed_args
                                      ? ($add_to_keys=1, '#%')
                                      : croak("constructor has no keyword arguments")
@@ -249,9 +251,9 @@ sub import {
                       if (defined (my $default_arg=delete $options{default})) {
                          $init_deferred ||= $default_arg =~ /(?<!\\)\$this\b/;
                          if ($key_in_expr) {
-                            ($deflt=$expr) =~ s/\#%/'$aliases[0]', $default_arg/g;
+                            $deflt = $expr =~ s/\#%/'$aliases[0]', $default_arg/gr;
                          } else {
-                            $deflt=$default_arg;
+                            $deflt = $default_arg;
                          }
                       }
                       my $val="\$kw[2*$index+1]";
@@ -524,7 +526,7 @@ _#_6_#_
       define_function($symtab, ".defined", sub { "$file line $line" });
       define_function($symtab, ".constructor", sub { ($cnt, $constructor, $constructor_deferred, $merger) });
       define_function($symtab, "sizeof", sub { $cnt });
-      define_function($symtab, ".signature", sub { ($signature, $min_arg, $max_arg, $trailing_arg) }) if $own_signature;
+      define_function($symtab, ".signature", sub { ($signature, $min_arg, $max_arg, $trailing_arg) }) if defined($own_signature);
       if ($keys_changed) {
          define_function($symtab, ".keys", sub { \%keys });
       }
