@@ -1,6 +1,7 @@
-#  Copyright (c) 1997-2015
-#  Ewgenij Gawrilow, Michael Joswig (Technische Universitaet Berlin, Germany)
-#  http://www.polymake.org
+#  Copyright (c) 1997-2019
+#  Ewgenij Gawrilow, Michael Joswig, and the polymake team
+#  Technische Universität Berlin, Germany
+#  https://polymake.org
 #
 #  This program is free software; you can redistribute it and/or modify it
 #  under the terms of the GNU General Public License as published by the
@@ -14,8 +15,9 @@
 #-------------------------------------------------------------------------------
 
 use strict;
-
 use namespaces;
+use warnings qw(FATAL void syntax misc);
+
 package Polymake;
 
 sub min($$) {  $_[0]<$_[1] ? $_[0] : $_[1]  }
@@ -63,10 +65,10 @@ sub delete_from_list {  # \@list, numeric scalar or ref => true if deleted
    for (my $i=$#$list; $i>=0; --$i) {
       if ($list->[$i]==$item) {
          splice @$list, $i, 1;
-         return 1;
+         return true;
       }
    }
-   0
+   false
 }
 ####################################################################################
 sub delete_string_from_list {   # \@list, string scalar => true if deleted
@@ -74,26 +76,26 @@ sub delete_string_from_list {   # \@list, string scalar => true if deleted
    for (my $i=$#$list; $i>=0; --$i) {
       if ($list->[$i] eq $item) {
          splice @$list, $i, 1;
-         return 1;
+         return true;
       }
    }
-   0
+   false
 }
 ####################################################################################
 sub contains {          # \@list, numeric scalar or ref => bool
    my ($list, $item)=@_;
    foreach my $elem (@$list) {
-      $elem==$item and return 1;
+      $elem==$item and return true;
    }
-   return 0;
+   return false;
 }
 ####################################################################################
 sub contains_string {           # \@list, scalar => bool
    my ($list, $item)=@_;
    foreach my $elem (@$list) {
-      $elem eq $item and return 1;
+      $elem eq $item and return true;
    }
-   return 0;
+   return false;
 }
 ####################################################################################
 # "string1", "string2" [, "delimiter" ]
@@ -133,9 +135,9 @@ sub equal_lists {
    my $end=@$l1;
    return 0 if $end!=@$l2;
    for (my $i=0; $i<$end; ++$i) {
-       $l1->[$i] == $l2->[$i] or return 0;
+       $l1->[$i] == $l2->[$i] or return false;
    }
-   1
+   true
 }
 
 sub equal_string_lists {
@@ -143,9 +145,9 @@ sub equal_string_lists {
    my $end=@$l1;
    return 0 if $end!=@$l2;
    for (my $i=0; $i<$end; ++$i) {
-       $l1->[$i] eq $l2->[$i] or return 0;
+       $l1->[$i] eq $l2->[$i] or return false;
    }
-   1
+   true
 }
 
 # \list1, start_index1, \list2, start_index2 => length of the equal sequence starting at given positions
@@ -187,7 +189,32 @@ sub equal_list_prefixes2 {
    }
    $ret
 }
+####################################################################################
+sub equal_hashes {
+   my ($h1, $h2)=@_;
+   if (keys(%$h1) == keys(%$h2)) {
+      while (my ($k, $v)=each %$h1) {
+         unless (exists $h2->{$k} and defined($v) ? $h2->{$k}==$v : !defined($h2->{$k})) {
+            keys %$h1;
+            return false;
+         }
+      }
+      true;
+   }
+}
 
+sub equal_string_hashes {
+   my ($h1, $h2)=@_;
+   if (keys(%$h1) == keys(%$h2)) {
+      while (my ($k, $v)=each %$h1) {
+         unless (exists $h2->{$k} and defined($v) ? $h2->{$k} eq $v : !defined($h2->{$k})) {
+            keys %$h1;
+            return false;
+         }
+      }
+      true;
+   }
+}
 ####################################################################################
 sub enforce_nl($) {
    $_[0].="\n" if substr($_[0],-1) ne "\n";
@@ -218,7 +245,7 @@ sub warn_print {
 sub croak {
    my ($pkg, $file, $line, $sub);
    my $i=0;
-   if (is_object($INC[0]) && defined($INC[0]->compile_scope)) {
+   if (is_object($INC[0]) && ref($INC[0]) eq "Polymake::Core::Application" && defined($INC[0]->compile_scope)) {
       do {
          ($pkg, $file, $line, $sub)=caller(++$i);
       } while ($pkg =~ /^Polymake(?:$|::Core::|::Overload\b|::Struct\b)/ || $sub =~ /::check_object_pkg$/ );
@@ -256,7 +283,6 @@ sub croak {
 
 sub beautify_error {
    unless ($DebugLevel) {
-      namespaces::temp_disable();
       use re 'eval';
       $@ =~ s/ at \(eval \d+\)(?:\[.*?:\d+\])? line 1(\.)?/$1/g;
       $@ =~ s/, <\$?$id_re> line \d+\.//go;
@@ -277,6 +303,16 @@ sub sanitize_help {
    mark_as_utf8string($_[0]);
 }
 ####################################################################################
+# this is a reduced version of Symbol::delete_package which does not delete the package itself
+# used by Configure and ShellHelpers
+sub wipe_package {
+   my ($stash)=@_;
+   foreach my $name (keys %$stash) {
+      undef *{$stash->{$name}};
+   }
+   %$stash=();
+}
+####################################################################################
 
 1
 
