@@ -1,6 +1,7 @@
-#  Copyright (c) 1997-2018
-#  Ewgenij Gawrilow, Michael Joswig (Technische Universitaet Berlin, Germany)
-#  http://www.polymake.org
+#  Copyright (c) 1997-2019
+#  Ewgenij Gawrilow, Michael Joswig, and the polymake team
+#  Technische Universität Berlin, Germany
+#  https://polymake.org
 #
 #  This program is free software; you can redistribute it and/or modify it
 #  under the terms of the GNU General Public License as published by the
@@ -15,19 +16,33 @@
 
 use strict;
 use feature 'state';
-use vars qw( $dirmode $group $permmask $root $ext_root $buildroot $config_file %ConfigFlags $buildmode );
+use vars qw( $dirmode $group $permmask $root $ext_root $buildroot $config_file %ConfigFlags $buildmode
+             $xsmod @callable @fakelibs );
 use POSIX qw ( :fcntl_h read write lseek );
 use File::Path;
 use Config;
 
 use Getopt::Long qw( GetOptions :config require_order no_ignore_case );
+
+sub collect_arglist {
+   my $list=shift;
+   @$list=@_;
+   while (@ARGV && $ARGV[0] !~ /^-/) {
+      push @$list, shift @ARGV;
+   }
+}
+
 unless (GetOptions( 'root=s' => \$root,
                     'extroot=s' => \$ext_root,
                     'buildroot=s' => \$buildroot,
                     'config=s' => \$config_file,
                     'mode=s' => \$buildmode,
                     'group=s' => \$group,
-                    'perms=i' => \$permmask ) &&
+                    'perms=i' => \$permmask,
+                    'xs=s' => \$xsmod,
+                    'callable=s' => sub { collect_arglist(\@callable, $_[1]) },
+                    'fakelibs=s' => sub { collect_arglist(\@fakelibs, $_[1]) } ) &&
+        !@ARGV &&
         defined($config_file) &&
         defined($root) &&
         defined($buildroot) &&
@@ -35,10 +50,11 @@ unless (GetOptions( 'root=s' => \$root,
         -f $config_file &&
         -d $root &&
         -d "$buildroot/$buildmode" &&
-        (!defined($ext_root) || -d $ext_root)) {
+        (defined($ext_root) ? -d $ext_root && !defined($xsmod) && !@callable && !@fakelibs
+                            : defined($xsmod) && -f $xsmod)) {
    die <<".";
 usage: $0 --root PATH --buildroot PATH --config PATH/config.ninja [ --extroot PATH --extconfig PATH/config.ninja ]
-          --buildmode Opt|Debug [ --group GROUP ] [ --perms MASK ] [ shared_module ... ]
+          --buildmode Opt|Debug [ --group GROUP ] [ --perms MASK ] [ --xsmod | --callable | --fakelib shared_module ... ]
 .
 }
 
@@ -66,12 +82,19 @@ $builddir="$buildroot/$buildmode";
 my $ext_name= $ext_root && basename($ext_root);
 
 my $destdir = $ENV{DESTDIR} // $ConfigFlags{DESTDIR};
+$destdir =~ s{/$}{};
 if (length($destdir)) {
    foreach ($InstallTop, $InstallInc, $InstallArch, $InstallBin, $InstallLib, $InstallDoc) {
-      substr($_,0,0).="$destdir/";
+      substr($_,0,0) .= $destdir;
    }
 }
 
+my $perlxpath="perlx/$Config::Config{version}/$Config::Config{archname}";
+
+if ($buildmode eq "San") {
+   load_sanitizer_flags();
+}
+
 if (defined $permmask) {
    if (($permmask & 0700) != 0700) {
       die "--perms must allow full access to the file owner\n";
@@ -89,8 +112,6 @@ if (defined $ext_root) {
 }
 
 sub install_core {
-   my $perlxpath="perlx/$Config::Config{version}/$Config::Config{archname}";
-
    make_dir($InstallTop);
    foreach my $subdir (qw(demo perllib resources scripts xml)) {
       copy_dir("$root/$subdir", "$InstallTop/$subdir", clean_dir => 1);
@@ -98,7 +119,7 @@ sub install_core {
 
    my $clean_dir=1;
    foreach my $file (qw(configure.pl.template generate_applib_fake.pl generate_ninja_targets.pl
-                        generate_ninja_targets.pl.template groom_wrappers.pl
+                        generate_ninja_targets.pl.template generate_cpperl_modules.pl
                         guarded_compiler.pl install.pl install.pl.template install_utils.pl rules.ninja)) {
       copy_file("$root/support/$file", "$InstallTop/support/$file", clean_dir => $clean_dir);
       $clean_dir=0;
@@ -157,22 +178,51 @@ sub install_core {
    }
 
    -d $InstallLib || make_dir($InstallLib);
-   foreach my $lib_file (@ARGV) {
-      if ($lib_file =~ m{($perlxpath/auto/.*\.$Config::Config{dlext})$}o) {
-         copy_file($lib_file, "$InstallArch/$1", mode => 0555, clean_dir => 1);
-      } elsif (-l $lib_file) {
-         copy_link($lib_file, "$InstallLib/".basename($lib_file));
-      } else {
-         copy_file($lib_file, "$InstallLib/".basename($lib_file), mode => 0555);
-      }
+
+   if (defined $xsmod) {
+      $xsmod =~ m{($perlxpath/auto/.*\.$Config::Config{dlext})$}o
+        or die "$0: path of perl extension module does not match the expected pattern\n";
+      copy_file($xsmod, "$InstallArch/$1", mode => 0555, clean_dir => 1);
    }
 
+   foreach my $lib_file (@callable) {
+      my $to=$InstallLib."/".basename($lib_file);
+      if (-l $lib_file) {
+         copy_link($lib_file, $to);
+      } else {
+         copy_file($lib_file, $to, mode => 0555, $^O eq "darwin" ? (lib_id => $to) : ());
+      }
+   }
    make_dir("$InstallArch/lib", clean_dir => 1);
    foreach my $app_dir (glob("$root/apps/*")) {
       my $app_mod=basename($app_dir).".$Config::Config{dlext}";
       copy_file("$builddir/lib/$app_mod", "$InstallArch/lib/$app_mod", mode => 0555);
    }
 
+   # These symlinks are used by the callable library bootstrap module.
+   # Any change in naming scheme must be reflected in Main.cc as well.
+   rel_symlink($InstallTop, "$InstallArch/shared");
+   if ($^O eq "darwin" && $ConfigFlags{FinkBase}) {
+      rel_symlink($ConfigFlags{FinkBase}, "$InstallArch/fink-base");
+   }
+
+   if (@fakelibs) {
+      my $stub_lib_name;
+      foreach my $lib_file (@fakelibs) {
+         my $basename=basename($lib_file);
+         my $to="$InstallArch/lib/$basename";
+         if (-l $lib_file) {
+            copy_link($lib_file, $to);
+            copy_link($lib_file, "$InstallLib/$basename");
+         } else {
+            $stub_lib_name //= $to;
+            copy_file($lib_file, $to, mode => 0555, $^O eq "darwin" ? (lib_id => $stub_lib_name) : ());
+            # This symlink is also used by the callable library bootstrap module.
+            rel_symlink($to, "$InstallLib/$basename");
+         }
+      }
+   }
+
    if (-f "$buildroot/doc/index.html") {
       copy_dir("$buildroot/doc", $InstallDoc, clean_dir => 1);
    }
@@ -245,14 +295,16 @@ sub copy_file {
    }
    if ($options{clean_dir}) {
       make_dir(dirname($to), clean_dir => 1);
+   } elsif (-d $to) {
+      $to .= "/" . basename($from);
    }
-   &copy;
+   copy($from, $to, %options);
 }
 
 sub copy_link {
    my ($from, $to)=@_;
    my $target=readlink($from);
-   if (-e $to) {
+   if (-e $to || -l $to) {
       unlink $to
         or die "can't remove old $to: $!\n";
    }
@@ -260,6 +312,27 @@ sub copy_link {
      or die "$0: can't create a symbolic link $to -> $target: $!\n";
 }
 
+sub rel_symlink {
+   my ($target, $link)=@_;
+   if (-e $link || -l $link) {
+      unlink $link
+        or die "$0: can't delete old $link: $!\n";
+   }
+   my $rel_link=$link;
+   my $common_prefix=1;
+   while ($rel_link =~ s{^(/[^/]+)(?=/)}{}) {
+      if ($common_prefix && substr($target, 0, length($1)) eq $1) {
+         $target=substr($target, length($1));
+      } else {
+         $common_prefix=0;
+         $target =~ s{^/?}{../};
+      }
+   }
+   $target =~ s{^/}{};
+   symlink $target, $link
+     or die "$0: can't create a symbolic link $link -> $target: $!\n";
+}
+
 sub compile_pattern {
    my $expr=join('|', map { "(?:^$_\$)" } @_);
    qr/$expr/;
@@ -334,6 +407,12 @@ sub copy {
       }
       POSIX::close $in;
       POSIX::close $out;
+
+      if (my $lib_id=$options{lib_id}) {
+         substr($lib_id, 0, length($destdir))="";
+         system("install_name_tool -id $lib_id $to")
+           and die "install_name_tool $to failed\n";
+      }
    } else {
       # text file: turn on write protection
       $mode=$fmode & 0555;
@@ -472,21 +551,21 @@ sub install_bin_scripts {
    close S;
 
    if ($^O eq "darwin" && $ConfigFlags{ARCHFLAGS} =~ /-arch /) {
-      s{^\#!\S+}{#!/usr/bin/arch $ConfigFlags{ARCHFLAGS} $^X}s;
+      s{^\#!/usr/bin/env perl}{#!/usr/bin/arch $ConfigFlags{ARCHFLAGS} $^X}s;
    } else {
-      s{^\#!\S+}{#!$Config::Config{perlpath}}s;
+      s{^\#!/usr/bin/env perl}{#!$Config::Config{perlpath}}s;
    }
 
    my $init_block=<<"---";
-   \$InstallTop="$ConfigFlags{InstallTop}";
-   \$InstallArch="$ConfigFlags{InstallArch}";
+   \$InstallTop='$ConfigFlags{InstallTop}';
+   \$InstallArch='$ConfigFlags{InstallArch}';
    \$Arch="$ConfigFlags{Arch}";
    \@BundledExts=qw(@BundledExts);
 ---
-   if ($ConfigFlags{FinkBase}) {
+   if ($^O eq "darwin" && $ConfigFlags{FinkBase}) {
       $init_block.="   \@addlibs=qw($ConfigFlags{FinkBase}/lib/perl5);\n";
    }
-   s|(^BEGIN\s*\{\s*\n)(?s:.*?)(^\}\n)|$1$init_block$2|m;
+   s/(^BEGIN\s*\{\s*\n)(?s:.*?)(^\}\n)/$1$init_block$2/m;
 
    if (-e "$InstallBin/polymake") {
       unlink "$InstallBin/polymake"
@@ -515,9 +594,9 @@ sub install_bin_scripts {
       s{^\#!\S+}{#!$Config::Config{perlpath}}s;
    }
 
-   s/=Version(?=;)/=$Version/;
-   s/=InstallTop(?=;)/="$ConfigFlags{InstallTop}"/;
-   s/=InstallArch(?=;)/="$ConfigFlags{InstallArch}"/;
+   s{=Version(?=;)}{=$Version};
+   s{=InstallTop(?=;)}{='$ConfigFlags{InstallTop}'};
+   s{=InstallArch(?=;)}{='$ConfigFlags{InstallArch}'};
 
    if (-e "$InstallBin/polymake-config") {
       unlink "$InstallBin/polymake-config"
@@ -532,25 +611,52 @@ sub install_bin_scripts {
      or die "chmod $InstallBin/polymake-config failed: $!\n";
 }
 
-sub transform_core_config_file {
+sub load_sanitizer_flags {
+   open my $BB, "<", "$builddir/build.ninja"
+      or die "can't read $builddir/build.ninja: $!\n";
+   while (<$BB>) {
+      chomp;
+      if (/^\s*PERLSANITIZE\s*=\s*/) {
+         $ConfigFlags{PERLSANITIZE}=$';
+      } elsif (/^\s*perlxpath\s*=\s*/) {
+         if ($perlxpath ne $') {
+            die "$builddir/build.ninja does not match the current running environment; expected perlxpath=$perlxpath\n";
+         }
+      } elsif (/^\s*buildroot\s*=\s*/) {
+         if ($buildroot ne $') {
+            die "$builddir/build.ninja does not match the current running environment; expected buildroot=$buildroot\n";
+         }
+      }
+   }
+   $ConfigFlags{PERLSANITIZE} or die "$builddir/build.ninja does not contain PERLSANITIZE flags\n";
+}
 
-   s|^\s* root \s*= \K .*|$ConfigFlags{InstallTop}|xm;
-   s|^\s* core\.includes \s*= \K .*|-I$ConfigFlags{InstallInc}|xm;
+sub transform_core_config_file {
+   s{^\s* configure\.command \s*= .* \n}{}xm;
+
+   s{^\s* root \s*= \K .*}{$ConfigFlags{InstallTop}}xm;
+   s{^\s* core\.includes \s*= \K .*}{-I$ConfigFlags{InstallInc}}xm;
 
    my $external_includes= $ConfigFlags{ExternalHeaders} =~ /\S/ ? "-I$ConfigFlags{InstallInc}/polymake/external" : "";
-   s|^\s* app\.includes \s*= \K .*|$external_includes|xm;
+   s{^\s* app\.includes \s*= \K .*}{$external_includes}xm;
 
-   s|^(?=\s*Arch\s*=)|PERL = $Config::Config{perlpath}\n|m;
-   s|^\s* BuildModes \s*= \K .*|$buildmode|xm;
-   s|^\s* DESTDIR \s*= .*$||xm;
+   s{^(?=\s*Arch\s*=)}{PERL = $Config::Config{perlpath}\n}m;
+   s{^\s* BuildModes \s*= \K .*}{$buildmode}xm;
+   s{^\s* (?: DESTDIR | CCACHE | CCWRAPPER ) \s*= .*$}{}xm;
+
+   s{[ =] -Wl,-z, \K now}{lazy}x;
+
+   if ($buildmode eq "San") {
+      s{^\s* PERL \s*= .*\n \K}{PERLSANITIZE = $ConfigFlags{PERLSANITIZE}\n}xm;
+   }
 }
 
 sub transform_extension_config_file {
    
-   s|^\s* root \s*= \K .*|$ConfigFlags{InstallTop}|xm;
-   s|^\s* extroot \s*= \K .*|$ConfigFlags{InstallTop}/ext/$ext_name|xm;
-   s|^\s* app\.includes \s*= \K .*|-I\${extroot}/include \${super.app.includes}|xm;
-   s|^\s* BuildModes \s*= \K .*|$buildmode|xm;
+   s{^\s* root \s*= \K .*}{$ConfigFlags{InstallTop}}xm;
+   s{^\s* extroot \s*= \K .*}{$ConfigFlags{InstallTop}/ext/$ext_name}xm;
+   s{^\s* app\.includes \s*= \K .*}{-I\${extroot}/include \${super.app.includes}}xm;
+   s{^\s* BuildModes \s*= \K .*}{$buildmode}xm;
 }
 
 # Local Variables:
